library httpserver;

import "dart:io" as io;
import "stream_binder.dart";
import "dart:async";

class HttpServer{
  
  final String filepath;
  StreamController _binder_controller = new StreamController();
  Stream<Binder> get onWebSocketConnection => _binder_controller.stream;
  HttpServer(String host,int port,this.filepath){
    io.HttpServer.bind(host, port).then(onHttpServer);
  }

  void onHttpServer(io.HttpServer server) {
    server.listen(onRequest);
  }
  void onRequest(io.HttpRequest request) {
    if (io.WebSocketTransformer.isUpgradeRequest(request)) {
      io.WebSocketTransformer.upgrade(request).then(_onWebSocket);
    } else {
      onHttp(request);
    }
  }
  
  void _onWebSocket(io.WebSocket ws){
    var binder = new Binder(ws,ws.add);
    _binder_controller.add(binder);
  }
  
  void onHttp(io.HttpRequest req) {
    
    if(req.uri.path.contains("..")){
      req.response
        ..statusCode = io.HttpStatus.BAD_REQUEST
        ..close()
        ;
    }
  
    var head = getHead(req.uri.path);
    var path = filepath + req.uri.path;
    if (req.uri.path == "/") {
      path += "index.html";
    }
    try {
      req.response.headers.add("content-type", head);
      new io.File(path).openRead().pipe(req.response).then((_) {}, onError: (e) {
        req.response.close();
      });
    } catch (e) {
      req.response.close();
    }
  
  }
  
  String getHead(String req) {
    try {
      if (req.endsWith("dart")) {
        return "application/dart";
      } else if (req.endsWith("css")) {
        return "text/css";
      } else if (req.endsWith("js")) {
        return "text/javascript";
      }
      return "text/html";
    } catch (e) {
      return "text/html";
    }
  }
}
